library(Hmisc)
library(memisc)
library(plyr)
library(dplyr)
library(reshape2)
library(stringr)
library(ggplot2)
library(scales)
library(foreach)
library(arm)
library(doParallel)
library(anesrake)

# set the number of cores to use for parallel computation
num_cores <- detectCores()
registerDoParallel(cores = min(10, max(num_cores - 2, 1)))

# set output directory
output_dir <- '../output/'

# election outcomes
final_obama <- .511
final_romney <- .472
final_obama_two_party <- final_obama / (final_obama + final_romney)

# set up ggplot theme
theme_set(theme_bw())
theme_update(legend.background = element_rect(colour="transparent", fill="transparent"),
	panel.grid.major = element_blank(),
	panel.grid.minor = element_blank()
)

# set up markers for the three presidential debates
add_debate_lines <- function(p) {
	p <- p + geom_vline(aes(xintercept=as.numeric(as.Date('2012-10-03'))), linetype='dotted')
	p <- p + geom_vline(aes(xintercept=as.numeric(as.Date('2012-10-16'))), linetype='dotted')
	p <- p + geom_vline(aes(xintercept=as.numeric(as.Date('2012-10-22'))), linetype='dotted')
	p
}

# categorize states by "contestedness"
battleground <- c('CO','FL','IA','NH','OH','VA')
quasi.battleground <- c('MI','MN','NC','NV','NM','PA','WI')
solid.obama <- c('CA','CT','DC','DE','HI','IL','ME','MD','MA','NJ','NY','OR','RI','VT','WA')
solid.romney <- c('AL','AK','AZ','AR','GA','ID','IN','KS','KY','LA','MS','MO','MT','NE','ND','OK','SC','SD','TN','TX','UT','WV','WY')

contestedness <- function(state) {
	result <- rep(NA, length(state))
	result[state %in% battleground] <- 'battleground'
	result[state %in% quasi.battleground] <- 'quasi-battleground'
	result[state %in% solid.obama] <- 'solid obama'
	result[state %in% solid.romney] <- 'solid romney'
	result
}

# capitalize first letter of every word in string
capitalize_all <- Vectorize(function(x) {
  s <- strsplit(x, " ")[[1]]
  paste(toupper(substring(s, 1,1)), substring(s, 2), sep="", collapse=" ")
})

inv_logit <- function(x) {
	val <- exp(x) / (1 + exp(x))
	val
}

# estimate the posterior predictive distirbution for a lmer model
# NOTE: only works for varying-intercept models
sim_predict_lmer <- function(fitted_model, new_data, num_sims=100) {
    ## cast data to a data.frame
    new_data <- as.data.frame(new_data)
	 
    ## generate posterior distribution for coeficients
    sims <- arm::sim(fitted_model, num_sims)
    
	 ## add up fixed effects
    fitted_values <- matrix(fixef(sims)[,"(Intercept)"], nrow=nrow(new_data), ncol=num_sims, byrow=T)
    for(fe in colnames(fixef(sims))) {
        if(fe != "(Intercept)" ) {
            alphas <- as.matrix(new_data[, fe]) %*% t(fixef(sims)[, fe])
            alphas[is.na(alphas)] <- 0
            fitted_values <- fitted_values + alphas
        }
    }
	 
    ## loop through the random effects (w/o date) and add up the coeficients
    for (category in names(ranef(sims)) ) {
        if (category != "finish_date") {
            ## generate the predictor names, to deal with interactions
            effect_components <- strsplit(category,':', fixed=T)[[1]]
            variable_names <- as.character(new_data[,effect_components[1]])
            if ( length(effect_components) >= 2 ) {
                for ( ndx in 2:length(effect_components) ) {
                    variable_names <- paste(variable_names, new_data[,effect_components[ndx]], sep=':')
                }
            }
				
            coefs <- ranef(sims)[[category]][,,'(Intercept)']
            ndx <- match(variable_names, colnames(coefs))
            betas <- t(coefs[,ndx])
            betas[is.na(betas)] <- 0
            rownames(betas) <- NULL
				
            ## add the betas to the estimate
            fitted_values <- fitted_values + betas
        }

    }
    fitted_values
}

sim_predict_logit <- function(fitted_model, new_data, num_sims=100) {
    fitted_values <- inv_logit(sim_predict_lmer(fitted_model, new_data, num_sims))
    fitted_values
}

# generate poststratified estimates for the nested vote intention model
poststratified_estimate <- function(major_party_model, obama_model, cells) {
	preds <- data.frame(
		weights = cells$p,
		major_party = predict(major_party_model, cells, type='response'),
		obama_given_major = predict(obama_model, cells, type='response')
	) 
	
	preds <- mutate(preds,
		other = 1 - major_party,
		obama = major_party * obama_given_major,
		romney = major_party * (1-obama_given_major)
	)

	result <- with(preds, data.frame(
		obama = sum(obama*weights),
		romney = sum(romney*weights),
		other = sum(other*weights)
	))
	result
}

# generate poststratified 95% CI for the nested vote intention model
poststratified_ci <- function(major_party_model, obama_model, cells, num_sims=100) {
	cells <- data.frame(cells)

	major_party = sim_predict_logit(major_party_model, cells, num_sims)
	obama_given_major = sim_predict_logit(obama_model, cells, num_sims)
	other = 1 - major_party
	obama = major_party * obama_given_major
	romney = major_party * (1-obama_given_major)

	categories <- c('sex', 'age', 'race', 'education', 'ideology', 'party_id', 'vote_2008','state_contestedness')
	mrp_estimates <- data.frame()
	for (category in c('full_pop', intersect(categories, names(cells)))) {
		if ( category == 'full_pop' ) {
			groups <- 'full_pop'
		} else {
			groups <- unique(cells[,category])
		}

		for (g in groups) {
			if (g == 'full_pop') {
				ndx <- rep(T, nrow(cells))
			} else {
				ndx <- cells[,category] == g
			}

			weights <- cells$weight[ndx] / sum(cells$weight[ndx])
			estimates <- data.frame(
				obama = t(obama[ndx,]) %*% weights,
				romney = t(romney[ndx,]) %*% weights,
				other = t(other[ndx,]) %*% weights
			)
			estimates <- transform(estimates,
				obama_two_party = obama/(obama + romney)
			)

			mrp_estimates <- with(estimates, rbind(mrp_estimates, data.frame(
				category = category,
				level = g,
				outcome = c('obama','romney','other','obama_two_party'),
				p = c( mean(obama), mean(romney), mean(other), mean(obama_two_party) ),
				std_dev = c( sd(obama), sd(romney), sd(other), sd(obama_two_party) ),
				lower = c( quantile(obama, 0.025), quantile(romney, 0.025), quantile(other, 0.025), quantile(obama_two_party, 0.025) ),
				upper = c( quantile(obama, 0.975), quantile(romney, 0.975), quantile(other, 0.975), quantile(obama_two_party, 0.975) )
			)))
		}
	}

	mrp_estimates

}


