source('library.R')
load('../data/private/xbox.RData')
load('../data/private/exit_poll.RData')

# limit to users with complete demographic info
user_demo <- user_demo %>%
   filter(complete.cases(user_demo)) %>%
	mutate(state_contestedness = contestedness(state))

# create a panel by limiting to users who submitted (complete) 
# demographic info before the first debate (Oct 3, 2012)
panel_demo <- filter(user_demo, ts < '2012-10-03')
	
# limit to voter intent responses between Sep 22 and Nov 5, during which time
# the intent question was asked daily
voter_intent <- filter(voter_intent, ts_date >= "2012-09-22" & ts_date <= "2012-11-05")

# mimic cross-sectional survey by limiting to the first response given by a user
voter_intent_cross_sectional <- user_demo %>%
   select(user_id, age, sex, race, education, state, state_contestedness, ideology, party_id, vote_2008) %>%
	inner_join(voter_intent, by='user_id') %>%
	group_by(user_id) %>%
	filter(rank(ts)==1)

# create panel responses by joining responses with the panelists
voter_intent <- panel_demo %>%
   select(user_id, age, sex, race, education, state, state_contestedness, ideology, party_id, vote_2008) %>%
	inner_join(voter_intent, by='user_id')

panel_demo <- filter(panel_demo, user_id %in% voter_intent$user_id)
num_panelists <- nrow(panel_demo)

exit_poll_2008 <- filter(exit_poll_2008, complete.cases(exit_poll_2008))
exit_poll_2012 <- filter(exit_poll_2012, complete.cases(exit_poll_2012))

#
# Fig 2: Comparison of Xbox and 2008 electorate demographics
#

attributes <- c('white','black','hispanic')
attributes <- c(attributes, 'male','female')
attributes <- c(attributes, '18-29','30-44','45-64','65+')
attributes <- c(attributes, 'barack obama','john mcCain','did not vote in 2008')
attributes <- c(attributes, 'liberal','moderate','conservative')
attributes <- c(attributes, 'democrat','republican')
attributes <- c(attributes, 'didn\'t graduate from HS', 'high school graduate', 'some college', 'college graduate')
attributes <- c(attributes, 'battleground','quasi-battleground','solid obama','solid romney')
attributes <- c(attributes, 'other','not sure')

categories <- c('sex','race','age','education','state_contestedness','party_id','ideology','vote_2008')
pretty_categories <- c('Sex','Race','Age','Education','State','Party ID','Ideology','2008 Vote')

xbox_demo_dist <- foreach (cat=categories, .combine='rbind') %do% {
	dist <- panel_demo %>% 
	   group_by_(cat) %>% 
		summarize(n = n()) %>%
		mutate(cat = cat, source = 'XBox     ') %>%
		dplyr::rename_(attribute = cat)
}

exit_2008_demo_dist <- foreach (cat=categories, .combine='rbind') %do% {
	dist <- exit_poll_2008 %>% 
	   group_by_(cat) %>% 
		summarize(n = sum(weight)) %>%
		mutate(cat = cat, source = '2008 Electorate') %>%
		dplyr::rename_(attribute = cat)
}

plot_data <- rbind(xbox_demo_dist, exit_2008_demo_dist) %>%
	filter(!is.na(attribute), attribute != 'did not vote in 2008', attribute != 'not sure') %>%
   mutate(
	   attribute = factor(attribute, levels=attributes),
	   cat = factor(mapvalues(cat, from=categories, to=pretty_categories), levels=pretty_categories),
		source = factor(source, levels=c('XBox     ','2008 Electorate'))
   ) %>%
	group_by(cat, source) %>%
	mutate(p = n / sum(n))
	
p <- ggplot(plot_data, aes(attribute, p, group=source)) + geom_point(aes(linetype=source))
p <- p + geom_line(aes(linetype=source))
p <- p + facet_grid(.~cat, scales="free_x", space="free")
p <- p + scale_y_continuous('', limits=c(0,1), labels=percent_format())
p <- p + scale_x_discrete('', labels=capitalize_all)
p <- p + theme(axis.text.x=element_text(angle=45, hjust=1, vjust=1))
p <- p + theme(legend.position='bottom', legend.title=element_blank())
ggsave(plot=p, file=paste0(output_dir,'demo_xbox_v_exit.pdf'), width=12, height=6)


#
# Fig 3b: Adjusted fraction of respondents that are democrats over time
#

# number of responses by group
response_table <- voter_intent %>%
   group_by(ts_date, sex, age, race, education, state) %>%
	summarize(
		num_dems = sum(party_id=='democrat'),
		num_reps = sum(party_id=='republican')
	)

# population composition, from the exit polls
exit_poll_2008_composition <- exit_poll_2008 %>%
   group_by(sex, age, race, education, state, ideology, party_id, vote_2008) %>%
	summarize(weight = sum(weight)) %>% 
	ungroup() %>%
	mutate(p = weight / sum(weight))
		
# generate the adjusted response distribution
dates <- unique(response_table$ts_date)
poststratified_response_rates <- foreach(d=dates, .combine='rbind') %dopar% {	
	model <- glmer(
		formula = cbind(num_dems, num_reps) ~ (1|sex) + (1|age) + (1|race) + (1|education) + (1|state), 
		data = filter(response_table, ts_date <= d & ts_date > d - 4),
		family = binomial()
	)
	
	# compute the poststratified estimates
	sims_by_group <- sim_predict_logit(model, exit_poll_2008_composition, 100) 
	estimates <- t(sims_by_group) %*% exit_poll_2008_composition$p
		
	# compute confidence interval
	result <- data.frame(
		ts_date = d,
		p = mean(estimates),
		std.dev = sd(as.vector(estimates)),
		lower = quantile(estimates, 0.025),
		upper = quantile(estimates, 0.975)
	)
	result
}

poststratified_response_rates <- poststratified_response_rates %>%
   filter(ts_date >= '2012-09-24') %>%
   arrange(ts_date)

# plot the figure
p <- ggplot(data=poststratified_response_rates, aes(ts_date, p)) 
p <- p + geom_line() + geom_point()
p <- p + scale_y_continuous('', labels=percent_format(), limits=c(.4,.6))
p <- p + scale_x_date('', labels = date_format("%b. %d"))
p <- p + geom_ribbon(aes(ymin=lower, ymax=upper), alpha=.3)
p <- p + geom_hline(aes(yintercept=.54), linetype='dashed')
p <- p + geom_hline(aes(yintercept=.5), linetype='solid')
p <- add_debate_lines(p)
p <- p + ggtitle('Two-party fraction of respondents who are Democrats\n')
ggsave(plot=p, file=paste0(output_dir, 'demo_adjusted_response_rates_smoothed.pdf'), width=6, height=4)

#
# Figure 3a and 4: Main model estimates
#

# compute smoothed estimates by aggregating all responses for 4 days before the given date
poststratified_estimates <- foreach (d=dates, .combine="rbind") %dopar% {

	ndx <- as.integer(voter_intent$ts_date - as.Date(d))

	# summarize data to speed up model fitting
	intent_table_demo <- voter_intent %>%
	   mutate(rel_date = as.integer(ts_date - as.Date(d))) %>%
		filter(rel_date > -4 & rel_date <= 0) %>% 
		group_by(sex, age, race, education, state) %>%
		summarize(
			obama = sum(response=='barack obama'),
		   romney = sum(response=='mitt romney'),
		   other = sum(response=='other')
		)
		
	intent_table_full <- voter_intent %>%
	   mutate(rel_date = as.integer(ts_date - as.Date(d))) %>%
		filter(rel_date > -4 & rel_date <= 0) %>% 
		group_by(sex, age, race, education, state, ideology, party_id, vote_2008) %>%
		summarize(
			obama = sum(response=='barack obama'),
		   romney = sum(response=='mitt romney'),
		   other = sum(response=='other')
		)	

	intent_table_full_cross <- voter_intent_cross_sectional %>%
	   mutate(rel_date = as.integer(ts_date - as.Date(d))) %>%
		filter(rel_date > -4 & rel_date <= 0) %>% 
		group_by(sex, age, race, education, state, ideology, party_id, vote_2008) %>%
		summarize(
			obama = sum(response=='barack obama'),
		   romney = sum(response=='mitt romney'),
		   other = sum(response=='other')
		)		

	# fit nested models
	major_party_demo_model <- glmer(
		formula = cbind(obama + romney, other) ~ (1|sex) + (1|age) + (1|race) + (1|education) + (1|state),
		data = intent_table_demo,
		family = binomial()
	)

	obama_demo_model <- glmer(
		formula = cbind(obama, romney) ~ (1|sex) + (1|age) + (1|race) + (1|education) + (1|state),
		data = filter(intent_table_demo, obama+romney > 0),
		family = binomial()
	)

	major_party_full_model <- glmer(
		formula = cbind(obama + romney, other) ~ (1|sex) + (1|age) + (1|race) + (1|education) + (1|state)
			+ (1|ideology) + (1|party_id) + (1|vote_2008),
		data = intent_table_full,
		family = binomial()
	)

	obama_full_model <- glmer(
		formula = cbind(obama, romney) ~ (1|sex) + (1|age) + (1|race) + (1|education) + (1|state)
			+ (1|ideology) + (1|party_id) + (1|vote_2008),
		data = filter(intent_table_full, obama+romney > 0),
		family = binomial()
	)
	
	major_party_full_cross_model <- glmer(
		formula = cbind(obama + romney, other) ~ (1|sex) + (1|age) + (1|race) + (1|education) + (1|state)
			+ (1|ideology) + (1|party_id) + (1|vote_2008),
		data = intent_table_full_cross,
		family = binomial()
	)

	obama_full_cross_model <- glmer(
		formula = cbind(obama, romney) ~ (1|sex) + (1|age) + (1|race) + (1|education) + (1|state)
			+ (1|ideology) + (1|party_id) + (1|vote_2008),
		data = filter(intent_table_full_cross, obama+romney > 0),
		family = binomial()
	)

	# compute the poststratified estimates
	estimates_demo <- poststratified_ci(major_party_demo_model, obama_demo_model, exit_poll_2008_composition)
	estimates_demo$ts_date = d
	estimates_demo$model = 'demo'

	estimates_full <- poststratified_ci(major_party_full_model, obama_full_model, exit_poll_2008_composition)
	estimates_full$ts_date = d
	estimates_full$model = 'full'
	
	estimates_full_cross <- poststratified_ci(major_party_full_cross_model, obama_full_cross_model, exit_poll_2008_composition)
	estimates_full_cross$ts_date = d
	estimates_full_cross$model = 'full_cross'

	result <- rbind(estimates_demo, estimates_full, estimates_full_cross)
	result
}

#
# compute the "house effect" (i.e., the overall skew in estimates after adjusting for demo,
# relative to adjusting for demo + party info)
#
intent_table_demo_alltime <- voter_intent %>%
	group_by(sex, age, race, education, state) %>%
	summarize(
		obama=sum(response=='barack obama'),
		romney=sum(response=='mitt romney'),
		other=sum(response=='other')
	)

intent_table_full_alltime <- voter_intent %>%
	group_by(sex, age, race, education, state, ideology, party_id, vote_2008) %>%
	summarize(
		obama=sum(response=='barack obama'),
		romney=sum(response=='mitt romney'),
		other=sum(response=='other')
	)	

major_party_demo_model <- glmer(
	formula = cbind(obama + romney, other) ~ (1|sex) + (1|age) + (1|race) + (1|education) + (1|state),
	data=intent_table_demo_alltime,
	family=binomial()
)

obama_demo_model <- glmer(
	formula = cbind(obama, romney) ~ (1|sex) + (1|age) + (1|race) + (1|education) + (1|state),
	data=filter(intent_table_demo_alltime, obama+romney > 0),
	family=binomial()
)

major_party_full_model <- glmer(
	formula = cbind(obama + romney, other) ~ (1|sex) + (1|age) + (1|race) + (1|education) + (1|state)
		+ (1|ideology) + (1|party_id) + (1|vote_2008),
	data=intent_table_full_alltime,
	family=binomial()
)

obama_full_model <- glmer(
	formula = cbind(obama, romney) ~ (1|sex) + (1|age) + (1|race) + (1|education) + (1|state)
		+ (1|ideology) + (1|party_id) + (1|vote_2008),
	data=filter(intent_table_full_alltime, obama+romney > 0),
	family=binomial()
)

demo_model_estimates <- poststratified_estimate(major_party_demo_model, obama_demo_model, exit_poll_2008_composition)
full_model_estimates <- poststratified_estimate(major_party_full_model, obama_full_model, exit_poll_2008_composition)

demo_house_effect <- data.frame(
	obama = demo_model_estimates[1,'obama'] - full_model_estimates[1,'obama'],
	romney = demo_model_estimates[1,'romney'] - full_model_estimates[1,'romney'],
	other = demo_model_estimates[1,'other'] - full_model_estimates[1,'other'],
	obama_two_party =
	( demo_model_estimates[1,'obama']/(demo_model_estimates[1,'obama'] + demo_model_estimates[1,'romney'])
	- full_model_estimates[1,'obama']/(full_model_estimates[1,'obama'] + full_model_estimates[1,'romney']) )
)

# adjust poststratified estimates for the house effect
poststratified_estimates <- mutate(poststratified_estimates,
	p = ifelse(model=='demo', p - as.numeric(demo_house_effect[1,as.character(outcome)]), p),
	lower = ifelse(model=='demo', lower - as.numeric(demo_house_effect[1,as.character(outcome)]), lower),
	upper = ifelse(model=='demo', upper - as.numeric(demo_house_effect[1,as.character(outcome)]), upper)
)

# plot results
plot_data <- filter(poststratified_estimates, 
	level=='full_pop' & outcome == 'obama_two_party' & ts_date >= '2012-09-24') %>%
	mutate(model = factor(mapvalues(model, 
		from=c('demo','full','full_cross'), 
		to=c('Demo model', 'Demo + party ID model', 'Cross-sectional model'))))

p <- ggplot(data=subset(plot_data, model=='Demo model'), aes(ts_date, p, group=outcome))
p <- p + geom_line(aes(linetype=outcome)) + geom_point()
p <- p + geom_ribbon(aes(ymin=lower, ymax=upper), alpha=.3)
p <- p + scale_y_continuous('', limits=c(.4,.6), labels=percent_format())
p <- p + scale_x_date('', labels = date_format("%b. %d"))
p <- p + theme(legend.position='none')
p <- p + geom_hline(aes(yintercept=final_obama_two_party), linetype='dashed')
p <- p + geom_hline(aes(yintercept=0.5), linetype='solid')
p <- add_debate_lines(p)
p <- p + ggtitle('Two-party Obama support\n')
ggsave(plot=p, file=paste0(output_dir, 'intent_ts_poststratified_by_demo_two_party.pdf'), width=6, height=4)

p <- ggplot(data=subset(plot_data, model %in% c('Demo + party ID model', 'Demo model')), 
	aes(ts_date, p, group=model))
p <- p + geom_ribbon(aes(ymin=lower, ymax=upper, alpha=model), fill='#999999')
p <- p + geom_line(aes(linetype=model, alpha=model)) + geom_point(aes(alpha=model))
p <- p + scale_y_continuous('', limits=c(.4,.6), labels=percent_format())
p <- p + scale_x_date('', labels = date_format("%b. %d"))
p <- p + theme(legend.position='none')
p <- p + scale_alpha_manual(values = c(.3,1))
p <- p + geom_hline(aes(yintercept=final_obama_two_party), linetype='dashed')
p <- p + geom_hline(aes(yintercept=0.5), linetype='solid')
p <- add_debate_lines(p)
p <- p + ggtitle('Two-party Obama support,\nadjusting for demographics (light line)\nor demographics and partisanship (dark line)\n')
ggsave(plot=p, file=paste0(output_dir, 'intent_ts_poststratified_two_party_overlayed.pdf'), width=6, height=4)

p <- ggplot(data=subset(plot_data, model %in% c('Demo + party ID model', 'Cross-sectional model')), 
	aes(ts_date, p, group=model))
p <- p + geom_ribbon(aes(ymin=lower, ymax=upper, alpha=model), fill='#999999')
p <- p + geom_line(aes(linetype=model, alpha=model)) + geom_point(aes(alpha=model))
p <- p + scale_y_continuous('', limits=c(.4,.6), labels=percent_format())
p <- p + scale_x_date('', labels = date_format("%b. %d"))
p <- p + theme(legend.position='none')
p <- p + scale_alpha_manual(values = c(1,.3))
p <- p + geom_hline(aes(yintercept=final_obama_two_party), linetype='dashed')
p <- p + geom_hline(aes(yintercept=0.5), linetype='solid')
p <- add_debate_lines(p)
p <- p + ggtitle('Two-party Obama support, based on panel (light line)\nor cross-sectional responses (dark line)\n')
ggsave(plot=p, file=paste0(output_dir, 'intent_ts_poststratified_two_party_cross.pdf'), width=6, height=4)

#
# Compare model predictions to actual outcomes for various demographic subgroups
#

intent_table <- voter_intent %>%
   mutate(rel_date = as.integer(ts_date - as.Date('2012-11-05'))) %>%
	filter(rel_date > -4 & rel_date <= 0) %>% 
	group_by(sex, age, race, education, state, ideology, party_id, vote_2008) %>%
	summarize(
		obama = sum(response=='barack obama'),
	   romney = sum(response=='mitt romney'),
	   other = sum(response=='other')
	)	

major_party_model <- glmer(
	formula = cbind(obama + romney, other) ~ (1|sex) + (1|age) + (1|race) + (1|education) + (1|state)
		+ (1|ideology) + (1|party_id) + (1|vote_2008),
	data = intent_table,
	family = binomial()
)

obama_model <- glmer(
	formula = cbind(obama, romney) ~ (1|sex) + (1|age) + (1|race) + (1|education) + (1|state)
		+ (1|ideology) + (1|party_id) + (1|vote_2008),
	data = filter(intent_table, obama+romney > 0),
	family = binomial()
)

election_predictions_by_group <- poststratified_ci(major_party_model, obama_model, exit_poll_2008_composition) %>%
	filter(category != 'full_pop' & category != 'vote_2008' & outcome == 'obama_two_party') %>%
	select(cat=category, attribute=level, p) %>%
	mutate(source='Xbox estimates     ')

categories <- c('sex','race','age','education','party_id','ideology')
pretty_categories <- c('Sex','Race','Age','Education','Party ID','Ideology')

election_2012_outcomes <- foreach (cat=categories, .combine='rbind') %do% {
	dist <- exit_poll_2012 %>% 
		filter(vote_2012 != 'other') %>%
	   group_by_(cat, 'vote_2012') %>% 
		summarize(n = sum(weight)) %>%
		mutate(cat = cat) %>%
		dplyr::rename_(attribute = cat)
}

election_2012_outcomes <- election_2012_outcomes %>%
   group_by(cat, attribute) %>%
	summarize(p = n[vote_2012=='barack obama'] / sum(n)) %>%
	mutate(source='Election outcomes')

plot_data <- rbind(election_predictions_by_group, election_2012_outcomes) %>%
   mutate(
	   attribute = factor(attribute, levels=attributes),
	   cat = factor(mapvalues(cat, from=categories, to=pretty_categories), levels=pretty_categories),
		source = factor(source, levels=c('Xbox estimates     ','Election outcomes'))
   )

p <- ggplot(plot_data, aes(attribute, p, group=source)) + geom_point(aes(linetype=source))
p <- p + geom_line(aes(linetype=source))
p <- p + facet_grid(.~cat, scales="free_x", space="free")
p <- p + scale_y_continuous('', limits=c(0,1), labels=percent_format())
p <- p + scale_x_discrete('', labels=capitalize_all)
p <- p + theme(axis.text.x=element_text(angle=45, hjust=1, vjust=1))
p <- p + theme(legend.position='bottom', legend.title=element_blank())
ggsave(plot=p, file=paste0(output_dir,'subgroup_predictions.pdf'), width=12, height=6)

#
# Fig 6: change in support before and after the debate
#

categories <- c('sex','race','age','education','party_id','ideology','vote_2008')
pretty_categories <- c('Sex','Race','Age','Education','Party ID','Ideology','2008 Vote')

debate_effect <- poststratified_estimates %>% 
   filter(outcome=='obama_two_party', model %in% c('demo','full')) %>%
	group_by(category, level, model) %>%
	summarize(
		support_change = p[ts_date=='2012-10-07'] - p[ts_date=='2012-10-02'],
		std_dev = sqrt(std_dev[ts_date=='2012-10-07']^2 + std_dev[ts_date=='2012-10-02']^2)
	) %>% ungroup()

plot_data <- debate_effect %>%
   filter(category %in% categories) %>%
	mutate( 
		level = factor(capitalize_all(as.character(level)), levels = capitalize_all(attributes)),
		category = factor(mapvalues(as.character(category), from=categories, to=pretty_categories), 
			levels=pretty_categories),
		model = mapvalues(as.character(model), from=c('demo','full'), 
			to=c('Adjusted by demographics     ','Adjusted by demographics and partisanship'))
	)

overall_debate_effect <- debate_effect %>%
	filter(level=='full_pop') %>%
	select(support_change)
	
p <- ggplot(data=plot_data, aes(level, -support_change))
p <- p + geom_errorbar(aes(ymin=-support_change-2*std_dev, ymax=-support_change+2*std_dev, width=0, fill=model),
	position=position_dodge(width=0.5), alpha=.9, show_guide=F)
p <- p + geom_point(aes(fill=model, shape=model), position=position_dodge(width=0.5))
p <- p + scale_shape_manual(values=c(21,21), guide = guide_legend(nrow=1))
p <- p + scale_fill_manual(values=c('white', 'black'))
p <- p + facet_grid(.~category, scales="free_x", space="free")
p <- p + scale_y_continuous('\nChange in two-party Obama support\n(positive values indicate a Romney gain)\n', 
	labels=percent_format())
p <- p + scale_x_discrete('')
p <- p + geom_hline(data=overall_debate_effect, aes(yintercept=-support_change), linetype='dashed')
p <- p + geom_hline(aes(yintercept=0), linetype='solid')
p <- p + theme(axis.text.x=element_text(angle=45, hjust=1, vjust=1),
	legend.position='bottom',
	legend.title=element_blank(),
	legend.text=element_text(size=14)
)
ggsave(plot=p, file=paste0(output_dir, 'debate_effect.pdf'), width=13, height=7)

#
# Fig 7: individual-level switching behavior around the debate
#

# construct an ad hoc panel comprised of users with at least 
# one response in the week immediately before the first debate, and at least one response 
# in the week immediately after
first_debate <- as.Date('2012-10-03')
switching_panel <- voter_intent %>%
	mutate(rel_date = as.integer(ts_date - first_debate)) %>%
	group_by(user_id) %>%
	filter(sum(rel_date >= -7 & rel_date < 0) >= 1, sum(rel_date > 0 & rel_date <= 7) >= 1) %>%
	select(user_id,ts,ts_date,rel_date,response)

# compute switching profiles for each user
switching_profiles <- switching_panel %>%
   group_by(user_id) %>%
	arrange(ts) %>%
	summarize(
		pre = rev(response[rel_date < 0])[1],
		post = response[rel_date > 0][1]
	) %>%
	inner_join(user_demo, by='user_id')


# post-stratify estimates of switching rates via four nested models

# set up outcomes for models
switching_profiles <- mutate(switching_profiles,
   pre_is_major_party = pre %in% c('barack obama', 'mitt romney'),
	switched = pre != post,
	switched_to_more_republican = ifelse(switched, 
		(pre == 'barack obama' & post == 'mitt romney') |
		(pre == 'other' & post == 'mitt romney') |
		(pre == 'mitt romney' & post == 'other'),
		NA)
	)

# model 1: p(pre == major.party)
model_pre_is_major_party <- glmer(
	pre_is_major_party ~ (1|sex) + (1|age) + (1|race) + (1|education) + (1|state)
			+ (1|ideology) + (1|party_id) + (1|vote_2008), 
	data = switching_profiles, 
	family = binomial()
)

# model 2: p(pre == obama | pre == major.party)
model_pre_is_obama_given_major_party <- glmer(
	(pre == 'barack obama') ~ (1|sex) + (1|age) + (1|race) + (1|education) + (1|state)
			+ (1|ideology) + (1|party_id) + (1|vote_2008), 
	data = filter(switching_profiles, pre_is_major_party), 
	family = binomial()
)

# model 3: p(switch)
model_switched <- glmer(
	switched ~ (1|sex) + (1|age) + (1|race) + (1|education) + (1|state) + (1|pre)
			+ (1|ideology) + (1|party_id) + (1|vote_2008), 
	data = switching_profiles, 
	family = binomial()
)

# model 4: p(switched to most popular alternative | switched)
model_switched_to_more_republican <- glmer(
	switched_to_more_republican ~ (1|sex) + (1|age) + (1|race) + (1|education) + (1|state) + (1|pre)
			+ (1|ideology) + (1|party_id) + (1|vote_2008), 
	data = filter(switching_profiles, switched), 
	family = binomial()
)

# generate estimates for each individual in the 2008 exit poll
switching_estimates <- exit_poll_2008_composition[rep(1:nrow(exit_poll_2008_composition),3),] %>%
   mutate(
		pre = factor(rep(c('barack obama','mitt romney','other'), each=nrow(exit_poll_2008_composition)), 
			levels=levels(switching_profiles$pre))
	)

prob_pre_is_major_party <- sim_predict_logit(model_pre_is_major_party, switching_estimates)
prob_pre_is_obama_given_major_party <- sim_predict_logit(model_pre_is_obama_given_major_party, switching_estimates)

prob_pre_obama <- prob_pre_is_major_party * prob_pre_is_obama_given_major_party
prob_pre_romney <- prob_pre_is_major_party * (1-prob_pre_is_obama_given_major_party)
prob_pre_other <- 1 - prob_pre_is_major_party

prob_examples <- prob_pre_obama
ndx <- switching_estimates$pre == 'mitt romney'
prob_examples[ndx, ] <- prob_pre_romney[ndx, ]
ndx <- switching_estimates$pre == 'other'
prob_examples[ndx, ] <- prob_pre_other[ndx, ]

prob_switched <- prob_examples * sim_predict_logit(model_switched, switching_estimates)
more_republican <- sim_predict_logit(model_switched_to_more_republican, switching_estimates)
prob_switched_to_more_republican <- prob_switched * more_republican
prob_switched_to_less_republican <- prob_switched * (1-more_republican)

ndx <- switching_estimates$pre == 'barack obama'
obama_to_romney <- as.vector( t(switching_estimates$p[ndx]) %*% prob_switched_to_more_republican[ndx,] )
obama_to_other <- as.vector( t(switching_estimates$p[ndx]) %*% prob_switched_to_less_republican[ndx,] )

ndx <- switching_estimates$pre == 'mitt romney'
romney_to_other <- as.vector( t(switching_estimates$p[ndx]) %*% prob_switched_to_more_republican[ndx,] )
romney_to_obama <- as.vector( t(switching_estimates$p[ndx]) %*% prob_switched_to_less_republican[ndx,] )

ndx <- switching_estimates$pre == 'other'
other_to_romney <- as.vector( t(switching_estimates$p[ndx]) %*% prob_switched_to_more_republican[ndx,] )
other_to_obama <- as.vector( t(switching_estimates$p[ndx]) %*% prob_switched_to_less_republican[ndx,] )

# plot results
plot_point_estimates <- data.frame(
	switch = c('Obama -> Romney','Obama -> Other','Romney -> Other',
		'Romney -> Obama','Other -> Romney', 'Other -> Obama'),
	p = c(mean(obama_to_romney), mean(obama_to_other), mean(romney_to_other), 
		mean(romney_to_obama), mean(other_to_romney), mean(other_to_obama)),
	std = c(sd(obama_to_romney), sd(obama_to_other), sd(romney_to_other), 
		sd(romney_to_obama), sd(other_to_romney), sd(other_to_obama))
)
	
plot_point_estimates <- mutate(plot_point_estimates, 
	switch = factor(as.character(switch), levels=as.character(switch)[order(p)]))
p <- ggplot(plot_point_estimates, aes(p, switch)) 
p <- p + geom_point()
p <- p + geom_errorbarh(aes(xmin=p-2*std, xmax=p+2*std, height=0))
p <- p + scale_x_continuous('',labels=percent_format(), limits=c(0,.015), expand=c(0,0))
p <- p + scale_y_discrete('')
ggsave(plot=p, file=paste0(output_dir, 'switching.pdf'), width=6, height=4)

#
# generate results based on raking
#

rake <- function(target_vars, intent=voter_intent, exit_poll=exit_poll_2008) {
	# construct target distribution
	targets <- list()
	for (cat in target_vars) {
		dist <- exit_poll %>% 
			group_by_(cat) %>% 
			summarize(weight=sum(weight)) %>%
			mutate(p=weight/sum(weight)) %>%
			dplyr::rename_(cat = cat)
		
		d <- dist$p
		names(d) <- as.character(dist$cat)
		targets[[cat]] <- d
	}
	
	# compute the raking estimates
	dates <- unique(intent$ts_date)
	results <- foreach (d=dates, .combine='rbind') %dopar% {
	   data_d <- intent %>%
			mutate(rel_date = as.integer(ts_date - as.Date(d))) %>%
			filter(rel_date > -4 & rel_date <= 0)
			
		r <- anesrake(targets, data_d, caseid=1:nrow(data_d), cap=100)
	
		estimate <- data_d %>% 
			mutate(weight=r$weightvec) %>%
			group_by(response) %>%
			summarize(weight=sum(weight)) %>%
			mutate(p=weight/sum(weight), ts_date=d) %>%
			select(ts_date, response, p)
		estimate
	}
	
	return(results)
}

raking_demo <- rake(c('age','sex','race','education'))
raking_demo$model <- 'demo'

raking_partisanship <- rake(c('age','sex','race','education','party_id','vote_2008'))
raking_partisanship$model <- 'party'

plot_data <- rbind(raking_demo, raking_partisanship) %>%
	filter(ts_date >= '2012-09-24', response != 'other') %>%
	group_by(ts_date, model) %>%
	summarize(p = p[response=='barack obama'] / sum(p)) %>%
	mutate(model=factor(model, levels=c('party','demo')))

p <- ggplot(data=plot_data, aes(ts_date, p))
p <- p + geom_line(aes(linetype=model)) + geom_point(aes(fill=model), shape=21)
p <- p + scale_fill_manual(values=c('black','white'))
p <- p + scale_y_continuous('', limits=c(.4,.6), labels=percent_format())
p <- p + scale_x_date('', labels = date_format("%b. %d"))
p <- p + theme(legend.position='none')
p <- p + geom_hline(aes(yintercept=final_obama_two_party), linetype='dashed')
p <- p + geom_hline(aes(yintercept=0.5), linetype='solid')
p <- add_debate_lines(p)
p <- p + ggtitle('Two-party Obama support,\nadjusting for demographics (dashed line)\nor demographics and partisanship (solid line)\n')
ggsave(plot=p, file=paste0(output_dir, 'raking_estimates.pdf'), width=6, height=4)

