source('library.R')

media_polls <- read.csv('../data/public/pollster_polls.csv')
media_polls <- mutate(media_polls,
	 startdate = as.Date(startdate, format = "%m/%d/%Y"),
	 startdate = as.Date(startdate, format = "%Y-%m-%d"),
	 enddate = as.Date(enddate, format = "%m/%d/%Y"),
	 enddate = as.Date(enddate, format = "%Y-%m-%d")
)

poll_v_party <- read.csv('../data/public/pollster_ID.csv', header=TRUE, 
   col.names=c('obama_diff','dem_diff','near_debate'))
poll_v_party <- mutate(poll_v_party, 
	obama_diff = obama_diff/100,
	dem_diff = dem_diff/100
)

# load Pew data
gallup_pew <- read.delim('../data/public/gallup_pew.tsv', header=FALSE, 
	col.names=c('source', 'date', 'p'))
gallup_pew <- mutate(gallup_pew, 
	p = p/100,
	date = as.Date(date, format = "%Y-%m-%d")
)
pew <- filter(gallup_pew, source=='Pew')

# load RAND data, collected by scraping the PDF image in Gutsche et al. (2014) paper
rand_romney <- read.csv('../data/public/rand_romney.csv', header=FALSE, col.names=c('date', 'p'))
rand_romney$candidate <- 'Romney'

rand_obama <- read.csv('../data/public/rand_obama.csv', header=FALSE, col.names=c('date', 'p'))
rand_obama$candidate <- 'Obama'

rand <- rbind(rand_obama, rand_romney)
rand <- mutate(
	rand, date = as.Date(date, format = "%Y/%m/%d"),
	p = as.numeric(as.character(p))/100
)

# restrict dates and drop spurious points, and average by day
rand <- filter(rand, date >= '2012-09-19' & date <= '2012-11-05' & p >= .4) %>%
   group_by(date, candidate) %>%
	summarize(p=mean(p))


#
# Fig 1a: media polls 
#

p <- ggplot(data=media_polls, aes(enddate, obama))
p <- add_debate_lines(p)
p <- p + geom_hline(aes(yintercept=.5), linetype='solid')
p <- p + geom_hline(aes(yintercept=final_obama_two_party), linetype='dashed')
p <- p + geom_point() + geom_smooth(color='black', method='loess', span=.25)
p <- p + scale_x_date('')
p <- p + scale_y_continuous('', limits=c(.4,.6), label=percent_format())
p <- p + theme(legend.position="none")
p <- p + ggtitle("Two-party Obama support\n")
ggsave(plot=p, file=paste0(output_dir, 'media_polls.pdf'), width=6, height=4)

#
# Fig 1b: relationship between poll estimates and party ID
#
 
p <- ggplot(data=poll_v_party, aes(dem_diff, obama_diff))
p <- p + geom_hline(yintercept=0, linetype = 5)
p <- p + geom_vline(xintercept=0, linetype = 5)
p <- p + scale_y_continuous("Change in Obama support\n", 
	label=percent_format(), limits=c(-.07,.07),
	breaks=seq(-.06,.06,.03))
p <- p + scale_x_continuous("\nChange in proportion Democrats", 
	label=percent_format(), limits=c(-.07,.07),
	breaks=seq(-.06,.06,.03))
p <- p + geom_point(aes(fill=near_debate), shape=21)
p <- p + scale_fill_manual(values=c('white', 'black'))
p <- p + geom_smooth(method='lm', se=FALSE, color='black', fullrange=TRUE)
p <- p + theme(legend.position="none")
ggsave(plot=p, file=paste0(output_dir, 'support_v_party.pdf'), width=4, height=4)


# Fig 5: RAND vs. Pew
rand <- rand %>% 
   group_by(date) %>%
	summarize(p = p[candidate=='Obama'] / (p[candidate=='Obama'] + p[candidate=='Romney'])) %>%
	mutate(source = 'Rand')

plot_data <- rbind(rand, pew)

p <- ggplot(data=plot_data, aes(date, p)) 
p <- add_debate_lines(p)
p <- p + geom_hline(aes(yintercept=.54), linetype='dashed')
p <- p + geom_hline(aes(yintercept=.5), linetype='solid')
p <- p + geom_line(aes(linetype=source)) + geom_point(aes(fill=source), shape=21)
p <- p + scale_linetype_manual(values=c('longdash','solid'))
p <- p + scale_fill_manual(values=c('white','black'))
p <- p + scale_x_date('')
p <- p + scale_y_continuous('', limits=c(.4,.6), label=percent_format())
p <- p + theme(legend.position='none')
p <- p + annotate(geom="text", x=as.Date('2012-10-09'), y=.532, label='RAND')
p <- p + annotate(geom="text", x=as.Date('2012-10-09'), y=.468, label='Pew')
p <- p + ggtitle('Two-party Obama support\n')
ggsave(plot=p, file=paste0(output_dir, 'rand_v_pew.pdf'), width=6, height=4)



